#region Using declarations
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Input;
using System.Windows.Media;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Gui;
using NinjaTrader.Gui.Chart;
using NinjaTrader.Gui.SuperDom;
using NinjaTrader.Gui.Tools;
using NinjaTrader.Data;
using NinjaTrader.NinjaScript;
using NinjaTrader.Core.FloatingPoint;
using NinjaTrader.NinjaScript.DrawingTools;
#endregion

//This namespace holds Indicators in this folder and is required. Do not change it. 
namespace NinjaTrader.NinjaScript.Indicators
{
    public class RenkoSignalBandsV3 : Indicator
    {
        protected override void OnStateChange()
        {
            if (State == State.SetDefaults)
            {
                Description = @"Renko Signal Bands V3";
                Name = "Renko Signal Bands V3";
                Calculate = Calculate.OnEachTick;
                IsOverlay = true;
                DisplayInDataBox = true;
                DrawOnPricePanel = true;
                DrawHorizontalGridLines = true;
                DrawVerticalGridLines = true;
                PaintPriceMarkers = true;
                ScaleJustification = NinjaTrader.Gui.Chart.ScaleJustification.Right;
                //Disable this property if your indicator requires custom values that cumulate with each new market data event. 
                //See Help Guide for additional information.
                IsSuspendedWhileInactive = true; MaximumBarsLookBack = MaximumBarsLookBack.Infinite;
                //
                HalfLength = 33;
                appPrice = RKSIG3_PRICE_SRC.WEIGHTED;
                BandsDeviations = 2.5;
                Interpolate = true;
                //
                alertOn = false;
                alertMessage = true;
                alertSound = true;
                soundFName = "alert1.wav";
                alertTitle = "Renko Signal Bands Alert";
                barMax = 5000;
                //
                AddPlot(new Stroke(Brushes.DodgerBlue, DashStyleHelper.Dash, 2), PlotStyle.Line, "TMBUF");
                AddPlot(new Stroke(Brushes.OrangeRed, DashStyleHelper.Dash, 2), PlotStyle.Line, "UPBUF");
                AddPlot(new Stroke(Brushes.LimeGreen, DashStyleHelper.Dash, 2), PlotStyle.Line, "DNBUF");
                AddPlot(new Stroke(Brushes.Aqua, 10), PlotStyle.TriangleUp, "UPARR");
                AddPlot(new Stroke(Brushes.Red, 10), PlotStyle.TriangleDown, "DNARR");
            }
            else if (State == State.Configure)
            {
                _onBarClose = Calculate == Calculate.OnBarClose;
                _kOnBar = _onBarClose ? 1 : 0;
                dtInd = DateTime.MinValue;
                dtCur = DateTime.MinValue;
                _nMax = barMax;// MaximumBarsLookBack == MaximumBarsLookBack.Infinite ? barMax : 256;
                ParamIni();
            }
            else if (State == State.DataLoaded)
            {
            }
        }

        private void ParamIni()
        {
            ///
            _sFName = @"\sounds\" + soundFName;
            dtAlert = DateTime.MinValue;
            //
            _minBars = HalfLength * 2 + 10;
            _kDrawMin = _minBars;
        }

        private int _kOnBar = 1, _minBars = 25, _kDrawMin = 5, _nMax;
        private string strEA = "RKSIG3_";
        private DateTime dtInd, dtCur;
        private bool _onBarClose = true;
        private readonly object _obLock0 = new object();
        //
        private int cInd, secTor = 1;
        private string lFont = "Arial";
        private SimpleFont txtFont;
        private DateTime dtAlert;
        private string _sFName = string.Empty;
        private int indIndex = 1;

        /// indicator
        double[] _open_, _high_, _low_, _close_, _vol_;
        DateTime[] _time_;
        double[] _tmBuf, _appsrc, _upBuf, _dnBuf, _wuBuf, _wdBuf, uparr, dnarr;
        void IndArrayIni(int _limit)
        {
            _open_ = new double[_limit];
            _high_ = new double[_limit];
            _low_ = new double[_limit];
            _close_ = new double[_limit];
            _time_ = new DateTime[_limit];
            _vol_ = new double[_limit];
            ///
            _tmBuf = new double[_limit];
            _appsrc = new double[_limit];
            _upBuf = new double[_limit];
            _dnBuf = new double[_limit];
            _wuBuf = new double[_limit];
            _wdBuf = new double[_limit];
            uparr = new double[_limit];
            dnarr = new double[_limit];
        }

        protected override void OnBarUpdate()
        {
            lock (_obLock0)
            {
                try
                {
                    if (CurrentBar <= _minBars || CurrentBar < Bars.Count - 1 - _kOnBar) return;
                    ///
                    bool bNewBar = Time[1].Subtract(dtInd).TotalSeconds > 0 || CurrentBar != cInd;
                    int ii, _gSz = Close.Count, _limit = Math.Min(_gSz - 8, _nMax);
                    int _kStart = bNewBar ? _limit : Math.Min(_limit, _kDrawMin);
                    bool bTickRef = DateTime.Now.Subtract(dtCur).TotalSeconds >= secTor;
                    if (bNewBar) IndArrayIni(_limit);
                    bool bIndCal = bNewBar || (!_onBarClose && bTickRef);
                    if (bIndCal)
                    {
                        dtCur = DateTime.Now;
                        for (ii = _kStart; ii >= 0; ii--)
                        {
                            if (ii >= _limit - 0 || ii >= _gSz - 0)
                                continue;
                            _close_[ii] = Close[ii];
                            _open_[ii] = Open[ii];
                            _high_[ii] = High[ii];
                            _low_[ii] = Low[ii];
                            _time_[ii] = Time[ii];
                            _vol_[ii] = Volume[ii];
                        }
                        f_fetchSrc(_kStart, _limit, _gSz,
                             ref _close_, ref _open_, ref _high_, ref _low_,
                             appPrice, ref _appsrc);
                        double FullLength = 2.0 * HalfLength + 1.0;
                        int jj = 0, kk = 0;
                        for (ii = _kStart; ii >= 0; ii--)
                        {
                            if (ii >= _limit - 0 || ii >= _gSz - 0)
                                continue;
                            double sum = (HalfLength + 1) * _appsrc[ii];
                            double sumw = (HalfLength + 1);
                            for (jj = 1, kk = HalfLength; jj <= HalfLength; jj++, kk--)
                            {
                                if (ii + jj < _limit && ii + jj < _gSz)
                                {
                                    sum += kk * _appsrc[ii + jj];
                                    sumw += kk;
                                }
                                if (jj <= ii)
                                {
                                    if (ii - jj >= 0)
                                    {
                                        sum += kk * _appsrc[ii - jj];
                                        sumw += kk;
                                    }
                                }
                            }
                            _tmBuf[ii] = sumw == 0 ? 0 : sum / sumw;
                            double diff = _appsrc[ii] - _tmBuf[ii];
                            if (ii == _limit - HalfLength - 1)
                            {
                                _upBuf[ii] = _tmBuf[ii];
                                _dnBuf[ii] = _tmBuf[ii];
                                if (diff >= 0)
                                {
                                    _wuBuf[ii] = Math.Pow(diff, 2);
                                    _wdBuf[ii] = 0;
                                }
                                else
                                {
                                    _wdBuf[ii] = Math.Pow(diff, 2);
                                    _wuBuf[ii] = 0;
                                }
                                continue;
                            }
                            if (ii < _limit - 1 && ii < _gSz - 1)
                            {
                                if (diff >= 0)
                                {
                                    _wuBuf[ii] = (_wuBuf[ii + 1] * (FullLength - 1) + Math.Pow(diff, 2)) / FullLength;
                                    _wdBuf[ii] = _wdBuf[ii + 1] * (FullLength - 1) / FullLength;
                                }
                                else
                                {
                                    _wdBuf[ii] = (_wdBuf[ii + 1] * (FullLength - 1) + Math.Pow(diff, 2)) / FullLength;
                                    _wuBuf[ii] = _wuBuf[ii + 1] * (FullLength - 1) / FullLength;
                                }
                            }
                            _upBuf[ii] = _tmBuf[ii] + BandsDeviations * Math.Sqrt(_wuBuf[ii]);
                            _dnBuf[ii] = _tmBuf[ii] - BandsDeviations * Math.Sqrt(_wdBuf[ii]);
                        }
                        for (ii = _kStart; ii >= 0; ii--)
                        {
                            if (ii >= _limit - 1 || ii >= _gSz - 1) continue;
                            TMBUF[ii] = _tmBuf[ii];
                            UPBUF[ii] = _upBuf[ii];
                            DNBUF[ii] = _dnBuf[ii];
                            UPARR.Reset(ii);
                            DNARR.Reset(ii);
                            if (_high_[ii + 1] > _upBuf[ii + 1] &&
                                _close_[ii + 1] > _open_[ii + 1] &&
                                _close_[ii] < _open_[ii])
                            {
                                DNARR[ii] = _high_[ii];
                                dnarr[ii] = _high_[ii];
                            }
                            if (_low_[ii + 1] < _dnBuf[ii + 1] &&
                             _close_[ii + 1] < _open_[ii + 1] &&
                             _close_[ii] > _open_[ii])
                            {
                                UPARR[ii] = _low_[ii];
                                uparr[ii] = _low_[ii];
                            }
                        }
                        //
                        if (bNewBar)
                        {
                            //
                            DateTime _tmSig = _time_[1];
                            dtAlert = (dtAlert == DateTime.MinValue) ? _tmSig : dtAlert;
                            if (alertOn)
                            {
                                if (_tmSig.Subtract(dtAlert).TotalSeconds > 0)
                                {
                                    if (uparr[indIndex] > 0)
                                    {
                                        dtAlert = _tmSig;
                                        Alert(alertTitle, Priority.High,
                                                    alertMessage ? "UP Signal" : string.Empty,
                                                    alertSound ? NinjaTrader.Core.Globals.InstallDir + _sFName : String.Empty,
                                                    15, Brushes.Transparent,
                                                    Brushes.Aqua);
                                    }
                                    else if (dnarr[indIndex] > 0)
                                    {
                                        dtAlert = _tmSig;
                                        Alert(alertTitle, Priority.High,
                                                    alertMessage ? "DOWN Signal" : string.Empty,
                                                    alertSound ? NinjaTrader.Core.Globals.InstallDir + _sFName : String.Empty,
                                                    15, Brushes.Transparent,
                                                    Brushes.Red);
                                    }
                                }
                            }
                        }
                    }
                    dtInd = Time[1];
                    cInd = CurrentBar;
                }
                //catch (Exception e) { Print("OnBarUpdate: " + e.ToString()); }
                catch { }
            }

        }

        void f_fetchSrc(int _kStart, int _limit, int _gSz,
            ref double[] _close, ref double[] _open, ref double[] _high, ref double[] _low,
            RKSIG3_PRICE_SRC _src, ref double[] _ret)
        {
            for (int ii = _kStart; ii >= 0; ii--)
            {
                if (ii >= _limit - 0 || ii >= _gSz - 0)
                    continue;
                if (_src == RKSIG3_PRICE_SRC.OPEN)
                    _ret[ii] = _open[ii];
                if (_src == RKSIG3_PRICE_SRC.HIGH)
                    _ret[ii] = _high[ii];
                if (_src == RKSIG3_PRICE_SRC.LOW)
                    _ret[ii] = _low[ii];
                if (_src == RKSIG3_PRICE_SRC.CLOSE)
                    _ret[ii] = _close[ii];
                if (_src == RKSIG3_PRICE_SRC.MEDIAN)
                    _ret[ii] = (_high[ii] + _low[ii]) * 0.5;
                if (_src == RKSIG3_PRICE_SRC.TYPICAL)
                    _ret[ii] = (_high[ii] + _low[ii] + _close[ii]) / 3;
                if (_src == RKSIG3_PRICE_SRC.WEIGHTED)
                    _ret[ii] = (_high[ii] + _low[ii] + _close[ii] * 2) * 0.25;
            }
        }


        #region Properties

        [Browsable(false)]
        [XmlIgnore()]
        public Series<double> TMBUF
        {
            get { return Values[0]; }
        }
        [Browsable(false)]
        [XmlIgnore()]
        public Series<double> UPBUF
        {
            get { return Values[1]; }
        }
        [Browsable(false)]
        [XmlIgnore()]
        public Series<double> DNBUF
        {
            get { return Values[2]; }
        }
        [Browsable(false)]
        [XmlIgnore()]
        public Series<double> UPARR
        {
            get { return Values[3]; }
        }
        [Browsable(false)]
        [XmlIgnore()]
        public Series<double> DNARR
        {
            get { return Values[4]; }
        }

        [Range(1, int.MaxValue), NinjaScriptProperty]
        [Display(ResourceType = typeof(Custom.Resource), Name = "Half Length", GroupName = "NinjaScriptParameters", Order = 0)]
        public int HalfLength
        { get; set; }

        [Range(0, int.MaxValue), NinjaScriptProperty]
        [Display(ResourceType = typeof(Custom.Resource), Name = "Applied Price", GroupName = "NinjaScriptParameters", Order = 1)]
        public RKSIG3_PRICE_SRC appPrice
        { get; set; }

        [Range(0, double.MaxValue), NinjaScriptProperty]
        [Display(ResourceType = typeof(Custom.Resource), Name = "Bands Deviations", GroupName = "NinjaScriptParameters", Order = 2)]
        public double BandsDeviations
        { get; set; }

        [Range(0, 1), NinjaScriptProperty]
        [Display(ResourceType = typeof(Custom.Resource), Name = "Interporation", GroupName = "NinjaScriptParameters", Order = 3)]
        public bool Interpolate
        { get; set; }

        [Range(0, 1), NinjaScriptProperty]
        [Display(ResourceType = typeof(Custom.Resource), Name = "Enable alert?", GroupName = "NinjaScriptParameters", Order = 100)]
        public bool alertOn
        { get; set; }

        [Range(0, 1), NinjaScriptProperty]
        [Display(ResourceType = typeof(Custom.Resource), Name = "Message", GroupName = "NinjaScriptParameters", Order = 101)]
        public bool alertMessage
        { get; set; }

        [Display(ResourceType = typeof(Custom.Resource), Name = "Message title", GroupName = "NinjaScriptParameters", Order = 102)]
        public string alertTitle
        { get; set; }

        [Range(0, 1), NinjaScriptProperty]
        [Display(ResourceType = typeof(Custom.Resource), Name = "Sound", GroupName = "NinjaScriptParameters", Order = 103)]
        public bool alertSound
        { get; set; }

        [Display(ResourceType = typeof(Custom.Resource), Name = "Sound file name", GroupName = "NinjaScriptParameters", Order = 104)]
        public string soundFName
        { get; set; }

        [Range(50, int.MaxValue), NinjaScriptProperty]
        [Display(ResourceType = typeof(Custom.Resource), Name = "Max Bars To Calculate", GroupName = "NinjaScriptParameters", Order = 200)]
        public int barMax
        { get; set; }

        #endregion


    }
}


public enum RKSIG3_PRICE_SRC
{
    OPEN,
    HIGH,
    LOW,
    CLOSE,
    MEDIAN,
    TYPICAL,
    WEIGHTED,
}

#region NinjaScript generated code. Neither change nor remove.

namespace NinjaTrader.NinjaScript.Indicators
{
	public partial class Indicator : NinjaTrader.Gui.NinjaScript.IndicatorRenderBase
	{
		private RenkoSignalBandsV3[] cacheRenkoSignalBandsV3;
		public RenkoSignalBandsV3 RenkoSignalBandsV3(int halfLength, RKSIG3_PRICE_SRC appPrice, double bandsDeviations, bool interpolate, bool alertOn, bool alertMessage, bool alertSound, int barMax)
		{
			return RenkoSignalBandsV3(Input, halfLength, appPrice, bandsDeviations, interpolate, alertOn, alertMessage, alertSound, barMax);
		}

		public RenkoSignalBandsV3 RenkoSignalBandsV3(ISeries<double> input, int halfLength, RKSIG3_PRICE_SRC appPrice, double bandsDeviations, bool interpolate, bool alertOn, bool alertMessage, bool alertSound, int barMax)
		{
			if (cacheRenkoSignalBandsV3 != null)
				for (int idx = 0; idx < cacheRenkoSignalBandsV3.Length; idx++)
					if (cacheRenkoSignalBandsV3[idx] != null && cacheRenkoSignalBandsV3[idx].HalfLength == halfLength && cacheRenkoSignalBandsV3[idx].appPrice == appPrice && cacheRenkoSignalBandsV3[idx].BandsDeviations == bandsDeviations && cacheRenkoSignalBandsV3[idx].Interpolate == interpolate && cacheRenkoSignalBandsV3[idx].alertOn == alertOn && cacheRenkoSignalBandsV3[idx].alertMessage == alertMessage && cacheRenkoSignalBandsV3[idx].alertSound == alertSound && cacheRenkoSignalBandsV3[idx].barMax == barMax && cacheRenkoSignalBandsV3[idx].EqualsInput(input))
						return cacheRenkoSignalBandsV3[idx];
			return CacheIndicator<RenkoSignalBandsV3>(new RenkoSignalBandsV3(){ HalfLength = halfLength, appPrice = appPrice, BandsDeviations = bandsDeviations, Interpolate = interpolate, alertOn = alertOn, alertMessage = alertMessage, alertSound = alertSound, barMax = barMax }, input, ref cacheRenkoSignalBandsV3);
		}
	}
}

namespace NinjaTrader.NinjaScript.MarketAnalyzerColumns
{
	public partial class MarketAnalyzerColumn : MarketAnalyzerColumnBase
	{
		public Indicators.RenkoSignalBandsV3 RenkoSignalBandsV3(int halfLength, RKSIG3_PRICE_SRC appPrice, double bandsDeviations, bool interpolate, bool alertOn, bool alertMessage, bool alertSound, int barMax)
		{
			return indicator.RenkoSignalBandsV3(Input, halfLength, appPrice, bandsDeviations, interpolate, alertOn, alertMessage, alertSound, barMax);
		}

		public Indicators.RenkoSignalBandsV3 RenkoSignalBandsV3(ISeries<double> input , int halfLength, RKSIG3_PRICE_SRC appPrice, double bandsDeviations, bool interpolate, bool alertOn, bool alertMessage, bool alertSound, int barMax)
		{
			return indicator.RenkoSignalBandsV3(input, halfLength, appPrice, bandsDeviations, interpolate, alertOn, alertMessage, alertSound, barMax);
		}
	}
}

namespace NinjaTrader.NinjaScript.Strategies
{
	public partial class Strategy : NinjaTrader.Gui.NinjaScript.StrategyRenderBase
	{
		public Indicators.RenkoSignalBandsV3 RenkoSignalBandsV3(int halfLength, RKSIG3_PRICE_SRC appPrice, double bandsDeviations, bool interpolate, bool alertOn, bool alertMessage, bool alertSound, int barMax)
		{
			return indicator.RenkoSignalBandsV3(Input, halfLength, appPrice, bandsDeviations, interpolate, alertOn, alertMessage, alertSound, barMax);
		}

		public Indicators.RenkoSignalBandsV3 RenkoSignalBandsV3(ISeries<double> input , int halfLength, RKSIG3_PRICE_SRC appPrice, double bandsDeviations, bool interpolate, bool alertOn, bool alertMessage, bool alertSound, int barMax)
		{
			return indicator.RenkoSignalBandsV3(input, halfLength, appPrice, bandsDeviations, interpolate, alertOn, alertMessage, alertSound, barMax);
		}
	}
}

#endregion
